//
//  BRLoadingView.m
//  NewsApp
//
//  Created by Basheer Malaa on 11/11/13.
//  Copyright (c) 2013 Basheer Malaa. All rights reserved.
//

#import "BRLoadingView.h"
#import "UIColor+iOS7Colors.h"
#import <QuartzCore/QuartzCore.h>

#define TEXT_MARGIN 10
static BRLoadingView *_brLoadingView = nil;



@interface BRLoadingView ()
@property (nonatomic , assign) BOOL hideLoading;
@end


@implementation BRLoadingView

- (id)initWithMessage:(NSString*)msg forView:(UIView*)view
{
    self = [self initWithFrame:view.bounds];
    if(self)
    {
        self.message = msg;
        [self generalInit];
        [view addSubview:self];
    }
    return self;
}
- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        //[self generalInit];
    }
    return self;
}

- (void)generalInit
{
    self.backgroundColor = [UIColor blackColor];
    self.alpha = 0.7;
    

    
    UILabel *lbl = [[UILabel alloc] initWithFrame:CGRectMake(0.0, 0.0, 0.0, 0.0)];
    [self addSubview:lbl];
    
    _lblText = lbl;
    self.lblText.font = [UIFont fontWithName:@"HelveticaNeue" size:15];
    self.lblText.text = self.message;
    
    CGSize textSize = [self.message sizeWithFont:self.lblText.font
                               constrainedToSize:CGSizeMake(320, 100)];
    
    self.lblText.frame = CGRectMake((TEXT_MARGIN+textSize.width) * -1,
                                    self.center.y,
                                    textSize.width + TEXT_MARGIN,
                                    textSize.height+ TEXT_MARGIN);
    

    self.lblText.backgroundColor = [UIColor whiteColor];
    self.lblText.textColor = [UIColor blackColor];
    self.lblText.textAlignment = NSTextAlignmentCenter;
    
    self.lblText.shadowOffset = CGSizeMake(0.0,0.0);
    self.lblText.shadowColor = [UIColor blackColor];

    self.lblText.layer.cornerRadius = 12;
    self.lblText.layer.shadowColor = [UIColor blackColor].CGColor;
    self.lblText.layer.shadowOffset = CGSizeMake(1, -3);
    self.lblText.layer.shadowOpacity = 1;
    self.lblText.opaque = NO;
    self.opaque = NO;
    self.lblText.alpha = 1;
    
    [self.lblText addObserver:self forKeyPath:@"text" options:0 context:NULL];
    self.hideLoading = NO;
    
    self.changeTextColorRandomly = YES;
}

+ (BRLoadingView*)presentWithMessage:(NSString*)message inView:(UIView*)view
{
    if(_brLoadingView)
    {
        if(_brLoadingView.superview == nil)
        {
            _brLoadingView = nil;
        }
        else
        {
            
            NSLog(@"Warning trying to present BRLoadingView above previously presented one");
            return _brLoadingView;
        }
    }
    
    _brLoadingView = [[BRLoadingView alloc] initWithMessage:message forView:view];
    [view bringSubviewToFront:_brLoadingView];
    return _brLoadingView;
}

+ (void)hideLoadingView
{
    _brLoadingView.hideLoading = YES;
}

+ (void)hideWithMessage:(NSString *)message
{
    _brLoadingView.message = message;
    _brLoadingView.hideLoading = YES;
    
}

+ (void)setMessage:(NSString *)message
{
    _brLoadingView.message = message;
}

- (void)setMessage:(NSString *)message
{
    _message = message;
}

- (void)resizeLabel
{
    CGSize textSize = [self.lblText.text sizeWithFont:self.lblText.font
                           constrainedToSize:CGSizeMake(320, 100)];
    textSize.width += TEXT_MARGIN;
    textSize.height += TEXT_MARGIN;
    
    CGRect labelRect = self.lblText.frame;
    labelRect.size = textSize;
    
    self.lblText.frame = labelRect;
}

- (void)hideLoadingView
{
    self.hideLoading = YES;
}

- (void)didMoveToSuperview
{
    if(self.superview)
    {
        // start animating label
        [self animateLabelWithMessage:self.message];
    }
    else
    {
        [_brLoadingView hideLoadingViewNow];
        _brLoadingView = nil;
    }
}

- (void)hideLoadingViewNow
{
    [UIView animateWithDuration:0.4 delay:0.1 options:UIViewAnimationOptionCurveEaseOut animations:^{
        _brLoadingView.alpha = 0.0;
    } completion:^(BOOL finished) {
        if(finished)
        {
            [_brLoadingView removeFromSuperview];
            _brLoadingView = nil;
        }
    }];
}

- (void)didAddSubview:(UIView *)subview
{
    NSLog(@"BRLoading did add subview");
}

- (void)animateLabelWithMessage:(NSString*)messsage
{
    CGFloat offsetFromCenter = 20;
    
    //1- animate fast to the middle
    [UIView animateWithDuration:0.3 delay:0.2 options:UIViewAnimationOptionCurveEaseOut animations:^{
        self.lblText.center = CGPointMake(self.frame.size.width/2,
                                          self.lblText.center.y);
    } completion:^(BOOL finished) {
       if(finished)
       {
           
           //2- animate slow from the middle
           [UIView animateWithDuration:0.8 animations:^{
               self.lblText.center = CGPointMake(self.lblText.center.x + offsetFromCenter,
                                                  self.lblText.center.y);
           } completion:^(BOOL finished) {
               if(finished)
               {
                   __block CGRect lblFrame = self.lblText.frame;
                   // 3- animate fast to the finish
                   [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
                            lblFrame.origin.x = self.frame.size.width;
                            self.lblText.frame = lblFrame;
                       
                   } completion:^(BOOL finished) {
                       if(finished)
                       {
                           [self changeTextColorRandomly];
                           
                           //4-  finish
                           if(self.hideLoading &&
                              [self.message isEqualToString:self.lblText.text])
                           {
                               [self hideLoadingViewNow];
                           }
                           else
                           {
                               [self randomizeTextColor];
                               self.lblText.text = self.message;
                               
                               
                               // recurisive call
                               lblFrame.origin.x = lblFrame.size.width * -1;
                               self.lblText.frame = lblFrame;
                               [self resizeLabel];
                               
                               [self animateLabelWithMessage:messsage];
                           }
                       }
                   }];
               }
            }];
       }
    }];
}


- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object
                        change:(NSDictionary *)change
                       context:(void *)context
{
   if([keyPath isEqualToString:@"text"])
   {
       CGSize size = [self.lblText.text sizeWithFont:self.lblText.font constrainedToSize:CGSizeMake(320, 40)];
       CGRect lblRect = self.lblText.frame;
       lblRect.size = CGSizeMake(size.width + TEXT_MARGIN*2,
                                 size.height + TEXT_MARGIN);
       self.lblText.frame = lblRect;
   }
}

- (void)randomizeTextColor
{
    if(self.changeTextColorRandomly)
    {
        self.lblText.textColor = [UIColor randomiOS7Color];
    }
}

#pragma mark - Dealloc

- (void)dealloc
{
    [self.lblText removeObserver:self forKeyPath:@"text"];
    _lblText = nil;
}
@end
